function output=threefunctiontypes(functiontype,nsim,nknots,nsubknots,doBayes)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DISCLAIMER: This software is provided as is with no implied warranty
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Author: Ciprian Crainiceanu
%Last edited: August 24, 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%This function generates input data (if needed) and calls the function
%spatiallyadaptivehetero which does the Bayesian analysis of spatially 
%adaptive P-splines with heteroscedastic errors
%
%                   y=m(x)+\epsilon
%
%where \epsilon is normal with non-constant variance 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%INPUT PARAMETERS
%
%
%inferencetype  =   Type if the inference: 1 = homoscedastic, 
%                    2 = heteroscedastic
%
%nsim           =   Number of iterations in the MCMC
%
%tuningparam    =   Variance of the proposed data in the
%                   Metropolis-Hastings step: 0.0001 for the univariate
%                   data, 0.1 for the multivariate data
%
%simID          =   SimulationID, so we can keep track of the number of
%                   simulations. It is also used when saving the generated
%                   data for future runs on the same data 
%
%generateinputdata = Flag to specify if the input data (x and y) need to be generated
%                        or data is already generated and needs to be
%                        loaded.
%                        1 = generate data and save it for future use
%                        2 = do not generate data but load it from the disk
%                        (inputdata_simID file)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%OUTPUT PARAMETERS
%
%output           =   structure containing smoothing elements
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Constant controlling the heterogeneity of function 2 and 3
%By default heterogeneity is small
jj=3;
degree_X=2;
tuningparam=0.001;

if (functiontype==1)
    x=linspace(0,1,1000)';
    mx=exp(-400*(x-0.6).^2)+5*exp(-500*(x-0.75).^2)/3+2*exp(-500*(x-0.9).^2);
    sigmaeps=0.5;
end


if (functiontype>1)
    if (functiontype == 2)
        jj=3;
        freq_j=2^((9-4*jj)/5);
        n=400;
        sigmaeps=0.2;
    end
    if (functiontype==3)
        jj=6;
        freq_j=2^((9-4*jj)/5);
        n=400;
        sigmaeps=0.2;
    end
    if (functiontype==4)
        jj=6;
        freq_j=0.05;
        n=2048;
        sigmaeps=1;
    end
    x=linspace(0,1,n)';
    mx=sqrt(x.*(1-x)).*sin(2*pi*(1+freq_j)./(x+freq_j));
end

y=normrnd(mx,sigmaeps*ones(size(mx,1),1));


%Obtain the frequentist adaptive fit using Ruppert's adaptive function
ruppert_est1 = srslocal(x,y,degree_X,nknots,nsubknots) ;
knots=ruppert_est1.knots;
subknots=ruppert_est1.subknots;

%Obtain the adaptive smoothing parameter using Ruppert's adaptive function
alpha_y = interp1(subknots,exp(ruppert_est1.alpha)',knots,'linear');

%beta and b are the parameters of the mean function
beta0=ruppert_est1.betaloc(1:degree_X+1);
b0=ruppert_est1.betaloc(degree_X+2:degree_X+nknots+1);

%Obtain the residuals and estimated constant variance
res_main=y-ruppert_est1.yhatloc;
AMSE_loc_frequentist=mean((ruppert_est1.yhatloc-mx).^2);
squared.residuals=res_main.^2;
hat_sigma2eps=mean(squared.residuals);
log_squared_residuals=log(squared.residuals+eps);
ruppert_est2 = srslocal(x,log_squared_residuals,degree_X,nknots,nsubknots) ;

sigma2epsilon0=exp(ruppert_est2.yhatgl);
gamma0=ruppert_est2.betagl(1:degree_X+1);
c0=ruppert_est2.betagl(degree_X+2:degree_X+nknots+1);
sigma2c0=ruppert_est2.sigma2hatgl/exp(ruppert_est2.alpha0);


%logtaub is used by mat2bugs
sigma2k0=hat_sigma2eps./alpha_y;

ruppert_est3=srslocal(knots,log(sigma2k0),1,nsubknots);

%delta, d are used by mat2bugs
delta0=ruppert_est3.betagl(1:2);
d0=ruppert_est3.betagl(3:2+nsubknots);

lambdad=exp(ruppert_est3.alpha0);
sigma2loc=ruppert_est3.sigma2hatgl;


%taud is used by mat2bugs
sigma2d0=ruppert_est3.sigma2hatgl/exp(ruppert_est3.alpha0);


CX = powerbasis01(x,degree_X,knots);
XX=CX(:,1:degree_X+1);
ZX=CX(:,degree_X+2:degree_X+nknots+1);

Cb=powerbasis01(knots,1,subknots);
Xb=Cb(:,1:2);
Zb=Cb(:,3:6);



initial_values=struct('beta0',beta0,'b0',b0,'gamma0',gamma0,...
'c0',c0,'delta0',delta0,'d0',d0,'sigma2c0',sigma2c0,'sigma2d0',sigma2d0,...
'sigma2epsilon0',sigma2epsilon0,'sigma2k0',sigma2k0);

AMSE_loc_Bayes=0;
if (doBayes==1)
%tic
spatiallyadaptiveheteroscedastic(y,XX,ZX,Xb,Zb,nsim,initial_values,tuningparam);
%toc
load beta0b0hetero
mean_Bayes_sim=CX*beta0b0hetero';
mean_Bayes=mean(mean_Bayes_sim');
AMSE_loc_Bayes=mean((mean_Bayes'-mx).^2);
end

output=struct('AMSE_loc_frequentist',AMSE_loc_frequentist,'AMSE_loc_Bayes',...
    AMSE_loc_Bayes,'x',x,'y',y,'mx',mx);