function current_values=spatiallyadaptiveheteroscedastic(Y,XX,ZX,Xb,Zb,nsim,initial_values,tuningparam)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DISCLAIMER: This software is provided as is with no implied warranty
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Author: Ciprian Crainiceanu
%Last edited: June 30, 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%This function does the Bayesian analysis of spatially adaptive P-splines
%with heteroscedastic errors
%
%
%                   y=m(x)+\epsilon
%where \epsilon is normal with variance depending on x and m(\cdot) is a 
%smooth function that can be highly variable in one region of the space and
%very smooth in other
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%INPUT PARAMETERS
%
%
%Y              =   Continuous response. An n dimensional vector
%
%XX             =   Matrix of fixed effects for the mean and variance functions
%
%ZX             =   Matrix of random effects for the mean and variance functions
%
%Xb             =   Matrix of fixed effects for the spatially adaptive variance
%                   of the truncated polynomial parameters
%
%Zb             =   Matrix of random effects for the spatially adaptive variance
%                   of the truncated polynomial parameters
%
%nsim           =   Number of simulations
%
%
%
%initial_values =   Initial values: beta0 (XX), b0 (ZX), gamma0 (XX), c0 (ZX),
%                   delta0 (Xb), d0 (Zb), sigma2c0 (c, \epsilon), sigma2d0 (d, Zb),
%                   sigma2epsilon0 (variance), sigma2k0 (shrinking process)
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%OUTPUT PARAMETERS
%
%current_values            =   structure containing the current values of
%                              the parameters being simulated
%
%beta0b0hetero             =    file containing the simulated values of the
%                               \beta and b parameters corresponding to the 
%                               mean function
%
%gamma0c0hetero            =    file containing the simulated values of the
%                               \gamma and c parameters corresponding to the 
%                               variance function
%
%delta0d0hetero            =    file containing the simulated values of the
%                               \delta and d parameters corresponding to the 
%                               variance function
%
%tauc0sigma2c0hetero       =    file containing the simulated values for
%                               \tau_c and \sigma^2_c controlling the
%                               amount of shrinkage of the c parameters
%
%taud0sigma2d0hetero       =    file containing the simulated values for
%                               \tau_d and \sigma^2_d controlling the
%                               amount of shrinkage of the d parameters
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Preparing the data for simulations
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
format long

n_observations=size(Y,1);       %Number of observations
CX=[XX ZX];                     %Full regression matrix for mean and variance
Cb=[Xb Zb];                     %Full regression matrix for spatially adaptive smoothing
sigma2u=0.01; sigmau=0.1;
sigma2v=0.01; sigmav=0.1;

Repsilon=chol(CX'*CX)/sigmau;
INVRepsilon=inv(Repsilon);
%Rb=chol(Cb'*Cb)/sigmav;
%INVRb=inv(Rb);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Number of columns:
psize=size(XX,2);               %Degree of the p-spline (mean and variance)              
Ksize=size(ZX,2);               %Number of knots (mean and variance)
qsize=size(Xb,2);               %Degree of p-spline (spatially adaptive process)
Kbsize=size(Zb,2);              %Number of knots (spatially adaptive process)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Initial values (0 will mean current value)
beta0=initial_values.beta0;     
b0=initial_values.b0; 
gamma0=initial_values.gamma0;
c0=initial_values.c0;
delta0=initial_values.delta0;
d0=initial_values.d0;
sigma2c0=initial_values.sigma2c0;
sigma2d0=initial_values.sigma2d0;
sigma2epsilon0=initial_values.sigma2epsilon0;
sigma2k0=initial_values.sigma2k0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Variances of fixed effects
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sigma2beta0=10^6;
sigma2gamma0=10^6;
sigma2delta0=10^6;
ac0=0.1;    bc0=10;
ad0=0.1;    bd0=10;
%ac0=10^(-6);    bc0=10^6;
%ad0=10^(-6);    bd0=10^6;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Do the simulations
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k=1:nsim
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Update the parameters of the mean function (\beta,b), Current values are
%beta0, b0 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

INV_SIGMA_epsilon=diag(1./sigma2epsilon0);              %Matrix of error process variances
INV_SIGMA_0X=diag(1./[sigma2beta0*ones(1,psize) sigma2k0']);

RX=chol(CX'*INV_SIGMA_epsilon*CX);                      %Obtain the Cholesky decomposition
invRX=inv(RX);

[UX,SX]=eig(invRX'*INV_SIGMA_0X*invRX);                 %Obtain the eigenvalue decomposition

diag_SX=diag(SX);                                       %Column vector
MX=invRX*UX*diag(1./(ones(psize+Ksize,1)+diag_SX))*UX'*invRX';

%Mean and standard deviation matrix of the parameters to simulate
muX=MX*CX'*INV_SIGMA_epsilon*Y;             
MX_one_half=invRX*UX*diag(1./sqrt(ones(psize+Ksize,1)+diag_SX));

theta_X=muX+MX_one_half*normrnd(zeros(psize+Ksize,1),ones(psize+Ksize,1));

%New simulation values of the parameters beta and b
beta0=theta_X(1:psize);
b0=theta_X(psize+1:psize+Ksize);


%Write in file beta0b0. The parameters appear on the same line
if (k==1)
    fout=fopen('beta0b0hetero','w');
else
    fout=fopen('beta0b0hetero','a');
end
for i=1:psize+Ksize  
      fprintf(fout,'%3.4f \t',theta_X(i));
end

fprintf(fout,'\n');

fclose(fout);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Update the parameters of the variance function (\gamma,c), Current values are
%gamma0, c0 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   

%Vector of log variances of the error process
Y_epsilon=log(sigma2epsilon0);  
INV_SIGMA_0epsilon=diag(1./[sigma2gamma0*ones(1,psize) sigma2c0*ones(1,Ksize)]);


[Uepsilon,Sepsilon]=eig(INVRepsilon'*INV_SIGMA_0epsilon*INVRepsilon);

diag_Sepsilon=diag(Sepsilon);                                       %Column vector

Mepsilon=INVRepsilon*Uepsilon*diag(1./(ones(psize+Ksize,1)+diag_Sepsilon))*Uepsilon'*INVRepsilon';   
    
%Mean and standard deviation matrix of the parameters to simulate
muepsilon=Mepsilon*CX'*Y_epsilon/sigma2u;     
Mepsilon_one_half=INVRepsilon*Uepsilon*diag(1./sqrt(ones(psize+Ksize,1)+diag_Sepsilon));   
    
theta_epsilon=muepsilon+Mepsilon_one_half*normrnd(zeros(psize+Ksize,1),ones(psize+Ksize,1));    

%Update the parameters of the variance process
gamma0=theta_epsilon(1:psize);
c0=theta_epsilon(psize+1:psize+Ksize);

%Write in file gamma0c0. The parameters appear on the same line
if (k==1)
    fout=fopen('gamma0c0hetero','w');
else
    fout=fopen('gamma0c0hetero','a');
end
for i=1:psize+Ksize  
      fprintf(fout,'%3.4f \t',theta_epsilon(i));
end

fprintf(fout,'\n');

fclose(fout);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Update the parameters of the shrinking parameters (\delta,d), Current values are
%delta0, d0 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Yb=log(sigma2k0);
INV_SIGMA_0b=diag(1./[sigma2delta0*ones(1,qsize) sigma2d0*ones(1,Kbsize)]);

%[Ub,Sb]=eig(INVRb'*INV_SIGMA_0b*INVRb);
%diag_Sb=diag(Sb);                                    %Column vector


Mb=inv(Cb'*Cb/sigma2v+INV_SIGMA_0b);

%Mean and standard deviation matrix of the parameters to simulate
mub=Mb*Cb'*Yb/sigma2v;     
thetab=mvnrnd(mub,Mb)';


%Update the parameters of the shrinkage parameter process
delta0=thetab(1:qsize);
d0=thetab(qsize+1:qsize+Kbsize);


%Write in file delta0d0. The parameters appear on the same line
if (k==1)
    fout=fopen('delta0d0hetero','w');
else
    fout=fopen('delta0d0hetero','a');
end
for i=1:qsize+Kbsize  
      fprintf(fout,'%3.4f \t',thetab(i));
end

fprintf(fout,'\n');

fclose(fout);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Update the variance of the truncated polynomial parameters describing the 
%P-spline for the variance of the error process: sigma^2_c
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


acplusK=ac0+Ksize/2;
bcc=1/(1/bc0+c0'*c0/2);

tauc0=gamrnd(acplusK,bcc);
sigma2c0=1/tauc0;

%Write in file tauc0sigma2c0
if (k==1)
    fout=fopen('tauc0sigma2c0hetero','w');
else
    fout=fopen('tauc0sigma2c0hetero','a');
end
 
fprintf(fout,'%3.4f \t',tauc0);
fprintf(fout,'%3.4f \t',sigma2c0);
fprintf(fout,'\n');

fclose(fout);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Update the variance of the truncated polynomial parameters describing the 
%P-spline for the shrinking parameter process: sigma^2_d
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


adplusKb=ad0+Kbsize/2;
bdd=1/(1/bd0+d0'*d0/2);

taud0=gamrnd(adplusKb,bdd);
sigma2d0=1/taud0;

%Write in file taud0sigma2d0
if (k==1)
    fout=fopen('taud0sigma2d0hetero','w');
else
    fout=fopen('taud0sigma2d0hetero','a');
end
 
fprintf(fout,'%3.4f \t',taud0);
fprintf(fout,'%3.4f \t',sigma2d0);
fprintf(fout,'\n');

fclose(fout);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Update the variances of the process. We use n one dimensional Metropolis
%Hastings updates. Proposal distributions are normal and the updates are
%done on the log of the variance not the variance itself
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

etaeta=CX*theta_epsilon;
mumu=CX*theta_X;
logsigma2epsilon0=log(sigma2epsilon0);

%This could be improved using matrix manipulations
for i=1:n_observations
    
    yc=logsigma2epsilon0(i);
    yn=normrnd(yc,tuningparam);
    localb=(Y(i)-mumu(i))^2;
    localc=etaeta(i);
    u_new_curr=-.5*(yn-yc)-.5*localb*(exp(-yn)-exp(-yc))-.5*(yn-localc)^2/sigma2u+...
    .5*(yc-localc)^2/sigma2u;
    accept_prob=min(1,exp(u_new_curr));
    decision=unifrnd(0,1);
    if (decision<accept_prob)
        yc=yn;
    end
    sigma2epsilon0(i)=exp(yc);
    
end %for i=1:n_observations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Update the shrinking parameters of the process. We use K one dimensional Metroplis
%Hastings updates. Proposal distributions are normal and the updates are
%done on the log of the variance not the variance itself
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

xixi=Cb*thetab;
b02b0=b0.^2;
logsigma2k0=log(sigma2k0);

for i=1:Ksize
    
    yc=logsigma2k0(i);
    yn=normrnd(yc,tuningparam);
    localb=b02b0(i);
    localc=xixi(i);
    u_new_curr=-.5*(yn-yc)-.5*localb*(exp(-yn)-exp(-yc))-.5*(yn-localc)^2/sigma2v+...
    .5*(yc-localc)^2/sigma2v;
    accept_prob=min(1,exp(u_new_curr));
    decision=unifrnd(0,1);
    if (decision<accept_prob)
        yc=yn;
    end
    sigma2k0(i)=exp(yc);
    
end %for i=Ksize


current_values=struct('beta0',beta0,'b0',b0,'gamma0',gamma0,...
'c0',c0,'delta0',delta0,'d0',d0,'sigma2c0',sigma2c0,'sigma2d0',sigma2d0,...
'sigma2epsilon0',sigma2epsilon0,'sigma2k0',sigma2k0);

end %for k=1:nsim
